"""
This package contains the validation functions for the metricspace package.
Helps ensure routines are expected and raises errors or warnings if undesired behavior is detected or expected. 

For example, if providing a search resolution window and not using optimizations, a warning will be raised indicating that the computation 
could take an astronomically long time. 

"""